use Test::More tests => 1;

BEGIN {
use_ok( 'CGI::Application::Plugin::Config::Context' );
}

diag( "Testing CGI::Application::Plugin::Config::Context $CGI::Application::Plugin::Config::Context::VERSION" );
